;; finder-site-lisp-generate.el -- generate new finder-inf.el

;; Copyright 2009, 2010, 2011, 2014, 2016 Kevin Ryde.

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Keywords: maint
;; URL: http://user42.tuxfamily.org/finder-site-lisp-el/index.html
;; EmacsWiki: FinderMode

;; This file is part of finder-site-lisp-el.
;;
;; finder-site-lisp-el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; finder-site-lisp-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with finder-site-lisp-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Usage: emacs -batch -no-site-file \
;;              -l finder-site-lisp-generate.el \
;; 	        -f finder-site-lisp-generate-batch
;;
;; Emacs `finder-compile-keywords' uses `insert-file-contents' so alas will
;; error out on broken symlinks.  Older code in xemacs has a readable check
;; (but which needs the nasty hack below!).

;;; Code:

(require 'finder)

(defun finder-site-lisp-generate-batch ()
  "Write a finder-inf.el file in the current directory."

  (message "finder-site-lisp-generate-batch ...")

  ;; Select coding system for emacs21 and emacs22.
  ;; Without this it prompts for the coding system when saving finder-inf.el.
  ;; iso-2022-7bit as used here is detected automatically when loaded, no
  ;; need for a "coding:" cookie.
  ;; This coding system should allow all chars likely to be encountered in
  ;; package descriptions.
  (find-file "finder-inf.el")
  (setq buffer-read-only nil)
  (erase-buffer)
  (setq buffer-file-coding-system 'iso-2022-7bit)

  ;; xemacs21 `finder-compile-keywords' tries to kill
  ;; "*finder-scratch*" even if it never created it
  (get-buffer-create "*finder-scratch*")

  ;; xemacs 21.4.22 `finder-compile-keywords' has a dubious call to
  ;; `file-readable-p' passing a filename without directory, so doesn't pick
  ;; up anything outside the current directory.  Try this nasty hack doing
  ;; the same sort of `(expand-file-name f d)' as used later in its code
  ;; when actually reading the file.
  ;;
  (let ((original-file-readable-p (symbol-function 'file-readable-p)))
    (fset 'file-readable-p
          (lambda (filename)
            (and (featurep 'xemacs)
                 (boundp 'f)
                 (boundp 'd)
                 (equal filename f)
                 ;; (message "%S" filename)
                 (funcall original-file-readable-p
                          (expand-file-name filename d)))))
    (unwind-protect
        (finder-compile-keywords)
      (fset 'file-readable-p original-file-readable-p)))

  ;; success
  (kill-emacs 0))

;; LocalWords: el symlinks

;;; finder-site-lisp-generate.el ends here
